﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Infragistics.Olap.FlatData;
using Infragistics.Win.UltraWinPivotGrid.UIElements;
using Infragistics.Win.UltraWinScrollBar;

namespace WPG_DataSetToFDS
{
    public partial class Form1 : Form
    {
        int lastPrintX;
        int lastPrintY;

        int originalWidth;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            DataTable table = GenerateData();
            IEnumerable<SportsData> data = CreateIEnumerable(table);
            FlatDataSource ds = InitializePivotData(data);

            var parameters = new CubeGenerationParameters();
            ds.GenerateCube(parameters);
            ds.InitializeAsync(ultraPivotGrid1);

            ultraPivotGrid1.DataSource = ds;
            olapDataSelector1.DataSource = ds;

            this.ultraPivotGrid1.ColumnAutoSizing = Infragistics.Win.UltraWinPivotGrid.ColumnAutoSizeFlags.Header;
            
            this.ultraPrintPreviewDialog1.Document = this.ultraPrintDocument1;

        }

        private DataTable GenerateData()
        {
            DataTable data = new DataTable();

            data.Columns.Add("Sport", typeof(string));
            data.Columns.Add("Nation", typeof(string));
            data.Columns.Add("City", typeof(string));
            data.Columns.Add("Budget", typeof(double));
            data.Columns.Add("Players", typeof(int));

            data.Rows.Add(new object[] { "Football", "England", "London", 31245, 5461});
            data.Rows.Add(new object[] { "Football", "England", "Manchester", 32514, 2134 });
            data.Rows.Add(new object[] { "Snooker", "England", "London", 12353, 5461 });
            data.Rows.Add(new object[] { "Tennis", "England", "London", 31245, 13532 });
            data.Rows.Add(new object[] { "Football", "France", "Paris", 5462, 12451 });
            data.Rows.Add(new object[] { "Tennis", "France", "Paris", 3151, 1245 });
            data.Rows.Add(new object[] { "Football", "Italy", "Milan", 6793, 5642 });
            data.Rows.Add(new object[] { "Volleyball", "Italy", "Rome", 31537, 3215 });
            data.Rows.Add(new object[] { "Basketball", "United States", "Miami", 3156, 16643 });
            data.Rows.Add(new object[] { "Basketball", "United States", "Los Angelees", 121467, 5461 });
            data.Rows.Add(new object[] { "Gymnastics", "China", "Beijing", 4123, 4789 });
            data.Rows.Add(new object[] { "Darts", "England", "London", 3129, 4314 });
            data.Rows.Add(new object[] { "Volleyball", "Bulgaria", "Sofia", 9690, 5431 });
            data.Rows.Add(new object[] { "Karate", "Japan", "Tokyo", 131254, 4231 });
            data.Rows.Add(new object[] { "Hockey", "Canada", "Toronto", 4325, 3123 });
            data.Rows.Add(new object[] { "Football", "Spain", "Madrid", 31551, 31235 });
            data.Rows.Add(new object[] { "Basketball", "Russia", "Moscow", 97473, 5461 });
            return data;
           
        }


        private FlatDataSource InitializePivotData(IEnumerable<SportsData> data)
        {
            IEnumerable<SportsData> itemSource = data;
            var settings = new FlatDataSourceInitialSettings();
            var ds = new FlatDataSource(itemSource, typeof(SportsData), settings);

            ds.InitializeHierarchyDescriptor += ds_InitializeHierarchyDescriptor;
            ds.InitializeMeasureDescriptor += ds_InitializeMeasureDescriptor;
            ds.AggregateMeasure += ds_AggregateMeasure;

            settings.Rows = "[SportsData].[Nation]";
            settings.Columns = "[SportsData].[Sport]";
            settings.Measures = "[Measures].[Budget]";

            return ds;
        }

        private IEnumerable<SportsData> CreateIEnumerable(DataTable table)
        {
            var listOfSportsData = table.Rows.Cast<DataRow>().Select(r => new SportsData(r.Field<string>("Sport"), 
                r.Field<string>("City"), r.Field<string>("Nation"), r.Field<double>("Budget"), 
                r.Field<int>("Players"))).ToList();
            return listOfSportsData;
        }

        private void ds_InitializeHierarchyDescriptor(object sender, InitializeHierarchyDescriptorEventArgs e)
        {
            LevelDescriptor level = e.HierarchyDescriptor.LevelDescriptors.ToList().FirstOrDefault(lvl => lvl.Name == "all");

            if (level != null)
            {
                switch (e.HierarchyDescriptor.PropertyDescriptor.Name)
                {
                    case "Nation":
                        level.MemberProvider = item => "All Nations";
                        break;
                    case "City":
                        level.MemberProvider = item => "All Cities";
                        break; 
                    case "Sport":
                        level.MemberProvider = item => "All Sports";
                        break; 
                }
            }

        }

        private void ds_InitializeMeasureDescriptor(object sender, InitializeMeasureDescriptorEventArgs e)
        {
            if ((e.MeasureDescriptor.Name == "Budget"))
            {
                e.MeasureDescriptor.DisplayFormat = "$#";
                e.MeasureDescriptor.Aggregation = MeasureAggregation.Sum;
            }
        }

        private void ds_AggregateMeasure(object sender, AggregateMeasureEventArgs e)
        {
            if (e.MeasureDescriptor.Name == "Budget")
            {
                double total = 0f;
                double count = 0f;
                foreach (object o in e.Items)
                {
                    dynamic item = o as SportsData;
                    if (item != null)
                    {
                        total += item.Budget;
                    }
                    count += 1f;
                }
                e.Value = total / count;
                e.Handled = true;
            }

        }

        private void button1_Click(object sender, EventArgs e)
        {
            ultraPrintPreviewDialog1.ShowDialog(this);
        }

        private void ultraPrintDocument1_BeginPrint(object sender, System.Drawing.Printing.PrintEventArgs e)
        {
            this.ultraPrintDocument1.DefaultPageSettings.Landscape = true;

            this.ultraPrintDocument1.Header.TextCenter = "Pivot Grid Print Preview";

            this.ultraPrintDocument1.DefaultPageSettings.Margins.Left = 50;
            this.ultraPrintDocument1.DefaultPageSettings.Margins.Right = 50;
            this.ultraPrintDocument1.DefaultPageSettings.Margins.Top = 50;
            this.ultraPrintDocument1.DefaultPageSettings.Margins.Bottom = 50;

            lastPrintX = 0;
            lastPrintY = 0;
        }

        private void ultraPrintDocument1_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
        {

            ExpandPivotIfNeeded(ultraPivotGrid1);
           
            Image image = GetImageFromControl(ultraPivotGrid1);

            RestorePivot(ultraPivotGrid1);

            Rectangle imageRect = new Rectangle(0, 0, image.Size.Width, image.Size.Height);
 
            e.Graphics.DrawImage(image, e.MarginBounds.X,
              e.MarginBounds.Y, imageRect, GraphicsUnit.Pixel);

        }

        private void RestorePivot(Infragistics.Win.UltraWinPivotGrid.UltraPivotGrid ultraPivotGrid1)
        {
            if (originalWidth != 0)
            {
                ultraPivotGrid1.Width = originalWidth;
                originalWidth = 0;
            }
        }

        private void ExpandPivotIfNeeded(Infragistics.Win.UltraWinPivotGrid.UltraPivotGrid ultraPivotGrid1)
        {
            int newWidth = 0;
           
            foreach (var column in ultraPivotGrid1.Columns)
            {
                newWidth += column.Width;
    
            }
            //the row header area is awlays drawn so we can use the UIElement
            var rowHeaderArea = ultraPivotGrid1.UIElement.GetDescendant(typeof(RowHeaderAreaUIElement));
            if (rowHeaderArea != null)
            {
                newWidth += rowHeaderArea.Rect.Width;
            }
            //the scrollbar is also always drawn so we can also use its UIElement
            var scrollBar = ultraPivotGrid1.UIElement.
                GetDescendant(typeof(ScrollBarUIElement),Orientation.Vertical);

            if(scrollBar != null)
            {
                newWidth += scrollBar.Rect.Width ;
            }
            
            if (newWidth > ultraPivotGrid1.Width)
            {
                originalWidth = ultraPivotGrid1.Width;
                ultraPivotGrid1.Width = newWidth;
            }
            
        }

        private Image GetImageFromControl(Control control)
        {
            

            Bitmap bmp = new Bitmap(control.Width,control.Height);

            control.DrawToBitmap(bmp, new Rectangle (0,0 ,control.Width, control.Height));

            return bmp;
        }

    }
}
